#############################################################################
# clive, video extraction utility
# Copyright (C) 2007 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  0.1.2-1307  USA
#############################################################################

import sys
import urllib2
import time
import traceback

from sys import stdout, stderr
from cStringIO import StringIO

from clive import *
from clive.newt import *
from clive.util import *


__all__ = ['CliveCLI']


class CliveCLI:
  """Command line interface for clive"""
  def __init__(self):
    sys.excepthook = self._exchook
    self.retval = 0

  def main(self):
    try:
      self.ext = Extract(
        self._onsay,
        self._onscan,
        self._onread,
        self._onprompt
      )

      # Show HTTP details
      agent = self.ext.opts.agent
      if not agent: agent = 'urllib-default'
      self._onsay('agent: %s  proxy: %s' % (agent, self.ext.opts.proxy))

      # Check flags
      if self.ext.opts.prefix:
        self._onsay('prefix: %s' % self.ext.opts.prefix)

      if self.ext.opts.newt:
        self.ext.opts.newt = _newt_installed

      if not self.ext.opts.verbose:
        self.ext.opts.newt = False

      # Scan for videos
      (videos,ignored) = self.ext.scan()
      self._onsay() # newline

      if len(videos) == 0:
        raise CliveError('error: nothing to extract')

      # Emit found video URLs to stdout
      if self.ext.opts.emit:
        for video in videos:
          self._onsay(video['url'], show_always=True)
        return # Exit program

      # Select videos (newt)
      if self.ext.opts.newt and self.ext.opts.verbose:
        # UI: NEWT
        ui = newtUI()
        if not self.ext.opts.autoyes:
          ui.main(videos, ignored)
          if not ui.result:
            raise CliveError('error: canceled')
          if len(ui.cursel) == 0:
            raise CliveError('error: nothing was selected')
          videos = ui.cursel
        if not self.ext.opts.overwrite and not self.ext.opts.skip_existing:
          if not ui.checkoverwrite(videos, self.ext.opts.prefix):
            raise CliveError('error: canceled')
      else:
        # UI: Plain cmdline -- print ignored URLs, video selection made later
        if len(ignored) > 0:
          text = 'show ignored urls (%d)? (y/N):' % len(ignored)
          a = self._onprompt(text)
          if a.lower() == 'y':
            for e in ignored:
              self._onsay('[ignored]: %s' % e['url'])
              self._onsay('[reason] : %s' % e['reason'])

      # Extract videos
      for video in videos:
        file = video['file']
        size = byteshuman(video['size'])
        self._onsay('video: %s (%s)' % (file,size),show_always=True)

        if self._promptvideo().lower() == 'y':
          self.ext.extract(video)

    except CliveError, e:
          self._onerror(e.text)

    # Exit gracefully, save configuration if requested
    if self.ext.opts.save:
      self.ext.optparser.saveconf()

  # Non-public

  def _onsay(self, text='\n', show_always=False):
    if self.ext.opts.verbose or show_always:
      stdout.write(self._cleanuptext(text))

  def _onerror(self, text):
    stderr.write(self._cleanuptext(text))
    self.retval = 1

  def _cleanuptext(self, text):
    if self.ext.opts.strip:
      text = self._striptext(text)
    return text.rstrip('\r\n') + '\n'

  def _striptext(self, text):
    """Strips text to fit in a line"""
    text = text[:70]
    if len(text) >= 70:
      text += ' (...)'
    return text

  def _onscan(self, text):
    if self.ext.opts.verbose:
      self._writebspaced('scan : '+text)

  def _onread(self, read, length):
    text_left = None

    if length == -1:
      text = 'read : %s' % bytesmb(read)
      progress = -1
    else:
      progress = percent(read,length)
      text = 'read : %s / %s (%03d%%)' % (
        bytesmb(read),
        bytesmb(length),
        progress
      )

    if read <= 1024:
      self.timestart = time.time()
    else:
      elapsed = time.time() - self.timestart
      rate = read / elapsed
      left = (length - read) / rate
      rate = byteshuman(long(rate))
      gmtime = time.gmtime(left)
      timeleft = '%02d:%02d:%02d' % (gmtime[3],gmtime[4],gmtime[5])
      text_left = '[%s] %12s/s' % (timeleft,rate)

    if text_left: text += (' '+text_left)
    if progress == 100: text += '\n'

    self._writebspaced(text)

  def _writebspaced(self, text):
    bspaces = '\b' * len(text)
    stdout.write(bspaces + text)
    stdout.flush()

  def _onprompt(self, text):
    text = '>> ' + text
    if self.ext.opts.verbose:
      return raw_input(text)
    else:
      return '(default)'

  def _promptvideo(self):
    a = 'y' # Default
    if not self.ext.opts.newt:
      if self.ext.opts.verbose:
        if not self.ext.opts.autoyes:
          a = self._onprompt('extract video? (y/N):')
    return a

  def _exchook(self, type, value, tb):
    self.retval = 1 # Set flag

    fmt  = '\nerror: %s'
    fmt2 = '\nerror: %s (%s)'

    if type == IOError:
      text = fmt % (value[1][1])
      self._onsay(text)
    elif type == urllib2.URLError or type == urllib2.HTTPError:
      text = fmt % (value)
      self._onsay(text)
    elif type != EOFError and type != KeyboardInterrupt:
      text = fmt2 % (type,value)
      self._onsay(text,show_always=True)
      a = self._onprompt('show traceback? (Y/n):')
      if a.lower() != 'n':
        trace = StringIO()
        traceback.print_exception(type,value,tb,None,trace)
        print trace.getvalue()
    else:
      print # new line, call print instead of .onsay(), --quiet may be used
