#############################################################################
# clive, video extraction utility
# Copyright (C) 2007 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  0.1.2-1307  USA
#############################################################################

import os
import ConfigParser

from optparse import OptionParser

from clive.version import *


__all__ = ['Options']


class Options:
  """Command line and runtime option parser"""
  CLIVE_CONF_FILE = '.cliverc'
  CLIVE_CONF_SECTION = 'clive'

  def __init__(self):
    self.cfile = os.path.join(os.path.expanduser('~'), self.CLIVE_CONF_FILE)

  def parse(self):
    self._parseconf()
    self._parseopts()
    return (self.opts, self.args, self.conf)

  def saveconf(self):
    self._setconf('agent',      self.opts.agent)    
    self._setconf('proxy',      self.opts.proxy)
    self._setconf('encoder',    self.opts.encoder)
    self._setconf('player',     self.opts.player)
    self.conf.write(open(self.cfile,'w'))

  # Non-public

  def _parseconf(self):
    self.conf = ConfigParser.SafeConfigParser()
    self.conf.read(self.cfile)

  def _parseopts(self):
    p = OptionParser(
      usage = PACKAGE_USAGE,
      description = PACKAGE_DESC,
      version = '%prog '+PACKAGE_VERSION+'\n'+PACKAGE_NOTICE
    )

    (n,v) = (PACKAGE_NAME,PACKAGE_VERSION)

    p.add_option('--overwrite','-o',
      dest='overwrite',
      action='store_true',
      default=False,
      help='overwrite existing files')
    p.add_option('--mpeg','-m',
      dest='mpeg',
      action='store_true',
      default=False,
      help='re-encode extracted to mpeg (see --encoder)')
    p.add_option('--avi','-a',
      dest='avi',
      action='store_true',
      default=False,
      help='re-encode extracted video to avi (see --encoder)')
    p.add_option('--flv','-f',
      dest='flv',
      action='store_true',
      default=False,
      help='re-encode extracted video to flv (see --encoder)')
    p.add_option('--save','-s',
      dest='save',
      action='store_true',
      default=False,
      help='save common options to runtime configuration')
    p.add_option('--yes','-y',
      dest='autoyes',
      action='store_true',
      default=False,
      help='extract video(s) without prompting')
    p.add_option('--play','-p',
      dest='play',
      action='store_true',
      default=False,
      help='play video (see also --player, --play-format)')
    p.add_option('--skip','-k',
      dest='skip_existing',
      action='store_true',
      default=False,
      help='skip already existing videos without prompting')
    p.add_option('--emit','-e',
      dest='emit',
      action='store_true',
      default=False,
      help='emit a listing of video extraction urls to stdout')
    p.add_option('--nostrip','-n',
      dest='strip',
      action='store_false',
      default=True,
      help='do not strip printed stdout status messages')
    p.add_option('--quiet','-q',
      dest='verbose',
      action='store_false',
      default=True,
      help='minimal stdout, do not prompt or overwrite')
    p.add_option('--nonewt',
      dest='newt',
      action='store_false',
      default=True,
      help='disable newt user interface')
    p.add_option('--play-format',
      dest='play_format',
      default=None,
      metavar='FORMAT',
      help='play re-encoded video FORMAT, e.g. --play-format=mpeg')
    p.add_option('--agent',
      dest='agent',
      default=None,
      metavar='AGENT',
      help='http request user-AGENT, e.g. --agent="%s/%s"' % (n,v))
    p.add_option('--proxy',
      dest='proxy',
      default={},
      metavar='PROXY',
      help='http PROXY for connections (url, "http_proxy" or None)')
    p.add_option('--player',
      dest='player',
      default=None,
      metavar='STRING',
      help='e.g. --player="/usr/bin/vlc %i"')
    p.add_option('--encoder',
      dest='encoder',
      default=None,
      metavar='STRING',
      help='e.g. --encoder="/usr/bin/ffmpeg -y -i %i %o"')
    p.add_option('--prefix',
      dest='prefix',
      default=None,
      metavar='PREFIX',
      help='save extracted videos in PREFIX')
    (self.opts, self.args) = p.parse_args()
    self._overrideconf()

  def _overrideconf(self):
    if not self.opts.agent:
      self.opts.agent = self._getconf('agent')
    if not self.opts.proxy:
      self.opts.proxy = self._getconf('proxy')
    if not self.opts.player:
      self.opts.player = self._getconf('player')
    if not self.opts.encoder:
      self.opts.encoder = self._getconf('encoder')

  def _getconf(self, name):
    try:
      value = self.conf.get(self.CLIVE_CONF_SECTION, name)
      return value.replace('%%','%')
    except:
      return None

  def _setconf(self, name, value):
    if not value or len(value) == 0 or value == 'None':
      try:
        self.conf.remove_option(self.CLIVE_CONF_SECTION,name)
      except:
        pass
      return

    try:
      value = value.replace('%','%%')
      self.conf.set(self.CLIVE_CONF_SECTION, name, value)
    except ConfigParser.NoSectionError:
      self.conf.add_section(self.CLIVE_CONF_SECTION)
      self._setconf(name,value)
    except:
      pass
