#############################################################################
# clive, video extraction utility
# Copyright (C) 2007 Toni Gundogdu
#
# clive is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  0.1.2-1307  USA
#############################################################################

import formatter

from htmllib import HTMLParser
from cStringIO import StringIO

from clive.util import *


__all__ = [
  'EmbedParser',
  'PageParser'
]


class EmbedParser(HTMLParser):
  """Parses HTML for <embed> tags"""
  def __init__(self):
    self.vurls = []
    fmt = formatter.AbstractFormatter(formatter.NullWriter())
    HTMLParser.__init__(self,fmt)

  def unknown_starttag(self, tag, attr):
    if tag.lower().find('embed') != -1:
      for a in attr:
        if len(a) > 1:
          if a[0] == 'src':
            self._appendurl(a[1])

  # Non-public

  def _appendurl(self, url):
    """Append URL to list if it qualifies"""
    
    if url.startswith('/'):
      # Treat URL as relative - insert host
      if self.title.find('YouTube -') != -1:
        url = 'http://www.youtube.com' + url
      elif self.title.find('- Google Video') != -1:
        url = 'http://video.google.com' + url
      else:
        raise CliveError('error: unknown host (tried to id from title)')

    # Convert embed to video page URL (if app.)
    url = normalize_url(url)

    # Ignore those that do not have YouTube/VGoogle 'signature'
    if url.find('/watch?v=') == -1:
      if url.find('/videoplay?') == -1:
        return

    for u in self.vurls:
      # Look for a duplicate
      if u == url: return

    self.vurls.append(url)


class PageParser:
  """Parses HTML for video output filename and extraction URL."""
  def parse(self, page, url):
    # Title -> filename
    fmt = formatter.AbstractFormatter(formatter.NullWriter())
    hp = HTMLParser(fmt)
    hp.feed(StringIO(page.data).read())
    hp.close()
    filename = parse_fname(hp.title)
    self.title = hp.title

    # Website specific video URL extraction
    if url.find('youtube.com') != -1:
      vurl = self._parseyt(page)
    elif url.find('video.google.com') != -1:
      vurl = self._parsegv(page,url)
    else:
      (host,path) = parse_url(url)
      raise CliveError('error: invalid host (%s)' % host)

    return (vurl,filename)

  def _parseyt(self, page):
    """
    Constructs video extraction URL.

    YouTube has a unique ID for each video on the website.
    These IDs can be extracted from the video page HTML,
    and be used with the 'get_video' (see below) to extract
    the actual flash video from the website.
    """
    text = parse_fromto(page.data,'player2.swf','"')

    if len(text) == 0:
      # Check if the page is flagged as 'mature content' and
      # terminate. Clive cannot handle login/cookies. Yet.
      if page.data.find('please verify you are 18 or older') != -1:
        raise CliveError('error: terminated (reason: age-verification)')
      if page.data.lower().find('this video is no longer available') != -1:
        raise CliveError('error: terminated (reason: video removed)')
      else:
        raise CliveError('error: video id not found in page html')

    return 'http://www.youtube.com/get_video?video_id=' + \
              text.split('video_id=')[1]
    
  def _parsegv(self, page, url):
    """
    Constructs video extraction URL.

    Google Video stores video details in gvp files,
    including the video extraction link. The URL to the
    gvp file can be found in the video page HTML.
    """
    if url.find('/videogvp/') == -1:
      # Extract gvp URL from HTML
      url = self._parsegvpurl(page)

    page.get(url)
    return parse_gvp(page.data, 'url:')

  def _parsegvpurl(self, page):
    """Extracts video gvp URL from HTML"""
    text = parse_fromto(page.data,'http://video.google.com/videogvp/','&')

    if len(text) == 0:
      raise CliveError('error: video gvp url not found in page html')

    return text
